@@ -31,6 +31,7 @@ class ResourceNotFoundError(LabelboxError):
 
     def __init__(self, db_object_type, params):
         """ Constructor.
+
         Args:
             db_object_type (type): A labelbox.schema.DbObject subtype.
             params (dict): Dict of params identifying the sought resource.
@@ -103,3 +104,12 @@ class MalformedQueryException(Exception):
 class UuidError(LabelboxError):
     """ Raised when there are repeat Uuid's in bulk import request."""
     pass
+
+
+class InconsistentOntologyException(Exception):
+    pass
+
+
+class MALValidationError(LabelboxError):
+    """Raised when user input is invalid for MAL imports."""
+    ...
