@@ -1,4 +1,5 @@
 import labelbox.schema.asset_metadata
+import labelbox.schema.asset_attachment
 import labelbox.schema.bulk_import_request
 import labelbox.schema.benchmark
 import labelbox.schema.data_row
@@ -13,3 +14,5 @@ import labelbox.schema.user
 import labelbox.schema.webhook
 import labelbox.schema.prediction
 import labelbox.schema.ontology
+import labelbox.schema.invite
+import labelbox.schema.role
