@@ -1,8 +1,11 @@
 from enum import Enum
+import logging
 
 from labelbox.orm.db_object import DbObject
 from labelbox.orm.model import Field
 
+logger = logging.getLogger(__name__)
+
 
 class AssetMetadata(DbObject):
     """ Asset metadata (AKA Attachments) provides extra context about an asset while labeling.
@@ -12,6 +15,11 @@ class AssetMetadata(DbObject):
         meta_value (str): URL to an external file or a string of text
     """
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        logger.warning(
+            "`AssetMetadata` is deprecated. Use `AssetAttachment` instead.")
+
     class MetaType(Enum):
         VIDEO = "VIDEO"
         IMAGE = "IMAGE"
