@@ -1,6 +1,7 @@
 import json
 import time
 from uuid import UUID, uuid4
+import functools
 
 import logging
 from pathlib import Path
@@ -9,6 +10,7 @@ import backoff
 import ndjson
 import requests
 from pydantic import BaseModel, validator
+from requests.api import request
 from typing_extensions import Literal
 from typing import (Any, List, Optional, BinaryIO, Dict, Iterable, Tuple, Union,
                     Type, Set)
@@ -113,6 +115,76 @@ class BulkImportRequest(DbObject):
     project = Relationship.ToOne("Project")
     created_by = Relationship.ToOne("User", False, "created_by")
 
+    @property
+    def inputs(self) -> List[Dict[str, Any]]:
+        """
+        Inputs for each individual annotation uploaded.
+        This should match the ndjson annotations that you have uploaded.
+
+        Returns:
+            Uploaded ndjson.
+
+        * This information will expire after 24 hours.
+        """
+        return self._fetch_remote_ndjson(self.input_file_url)
+
+    @property
+    def errors(self) -> List[Dict[str, Any]]:
+        """
+        Errors for each individual annotation uploaded. This is a subset of statuses
+
+        Returns:
+            List of dicts containing error messages. Empty list means there were no errors
+            See `BulkImportRequest.statuses` for more details.
+
+        * This information will expire after 24 hours.
+        """
+        self.wait_until_done()
+        return self._fetch_remote_ndjson(self.error_file_url)
+
+    @property
+    def statuses(self) -> List[Dict[str, Any]]:
+        """
+        Status for each individual annotation uploaded.
+
+        Returns:
+            A status for each annotation if the upload is done running.
+            See below table for more details
+
+        .. list-table::
+           :widths: 15 150
+           :header-rows: 1
+
+           * - Field
+             - Description
+           * - uuid
+             - Specifies the annotation for the status row.
+           * - dataRow
+             - JSON object containing the Labelbox data row ID for the annotation.
+           * - status
+             - Indicates SUCCESS or FAILURE.
+           * - errors
+             - An array of error messages included when status is FAILURE. Each error has a name, message and optional (key might not exist) additional_info.
+
+        * This information will expire after 24 hours.
+        """
+        self.wait_until_done()
+        return self._fetch_remote_ndjson(self.status_file_url)
+
+    @functools.lru_cache()
+    def _fetch_remote_ndjson(self, url: str) -> List[Dict[str, Any]]:
+        """
+        Fetches the remote ndjson file and caches the results.
+
+        Args:
+            url (str): Can be any url pointing to an ndjson file.
+        Returns:
+            ndjson as a list of dicts.
+        """
+        response = requests.get(url)
+        response.raise_for_status()
+        return ndjson.loads(response.text)
+
     def refresh(self) -> None:
         """Synchronizes values of all fields with the database.
         """
@@ -318,33 +390,29 @@ class BulkImportRequest(DbObject):
 
 def _validate_ndjson(lines: Iterable[Dict[str, Any]],
                      project: "labelbox.Project") -> None:
-    """   
+    """
     Client side validation of an ndjson object.
 
     Does not guarentee that an upload will succeed for the following reasons:
         * We are not checking the data row types which will cause the following errors to slip through
             * Missing frame indices will not causes an error for videos
-        * Uploaded annotations for the wrong data type will pass (Eg. entity on images) 
+        * Uploaded annotations for the wrong data type will pass (Eg. entity on images)
         * We are not checking bounds of an asset (Eg. frame index, image height, text location)
- 
+
     Args:
         lines (Iterable[Dict[str,Any]]): An iterable of ndjson lines
         project (Project): id of project for which predictions will be imported
-    
+
     Raises:
         MALValidationError: Raise for invalid NDJson
         UuidError: Duplicate UUID in upload
     """
-    data_row_ids = {
-        data_row.uid: data_row for dataset in project.datasets()
-        for data_row in dataset.data_rows()
-    }
     feature_schemas = get_mal_schemas(project.ontology())
     uids: Set[str] = set()
     for idx, line in enumerate(lines):
         try:
             annotation = NDAnnotation(**line)
-            annotation.validate_instance(data_row_ids, feature_schemas)
+            annotation.validate_instance(feature_schemas)
             uuid = str(annotation.uuid)
             if uuid in uids:
                 raise labelbox.exceptions.UuidError(
@@ -467,7 +535,7 @@ class SpecialUnion:
             raises:
                 KeyError: data does not contain the determinant fields for any of the types supported by this SpecialUnion
                 ValidationError: Error while trying to construct a specific object in the union
-            
+
         """
         if isinstance(data, BaseModel):
             data = data.dict()
@@ -524,12 +592,6 @@ class NDBase(NDFeatureSchema):
     uuid: UUID
     dataRow: DataRow
 
-    def validate_datarow(self, valid_datarows):
-        if self.dataRow.id not in valid_datarows:
-            raise ValueError(
-                f"datarow {self.dataRow.id} is not attached to the specified project"
-            )
-
     def validate_feature_schemas(self, valid_feature_schemas):
         if self.schemaId not in valid_feature_schemas:
             raise ValueError(
@@ -541,9 +603,8 @@ class NDBase(NDFeatureSchema):
                 f"Schema id {self.schemaId} does not map to the assigned tool {valid_feature_schemas[self.schemaId]['tool']}"
             )
 
-    def validate_instance(self, valid_datarows, valid_feature_schemas):
+    def validate_instance(self, valid_feature_schemas):
         self.validate_feature_schemas(valid_feature_schemas)
-        self.validate_datarow(valid_datarows)
 
     class Config:
         #Users shouldn't to add extra data to the payload
@@ -623,6 +684,7 @@ class NDBaseTool(NDBase):
 
     #This is indepdent of our problem
     def validate_feature_schemas(self, valid_feature_schemas):
+        super(NDBaseTool, self).validate_feature_schemas(valid_feature_schemas)
         for classification in self.classifications:
             classification.validate_feature_schemas(
                 valid_feature_schemas[self.schemaId]['classifications'])
@@ -632,14 +694,14 @@ class NDBaseTool(NDBase):
         #Create uuid and datarow id so we don't have to define classification objects twice
         #This is caused by the fact that we require these ids for top level classifications but not for subclasses
         results = []
+        dummy_id = 'child'.center(25, '_')
         for row in value:
-            row.update({
-                'dataRow': {
-                    'id': 'child'.center(25, '_')
+            results.append({
+                **row, 'dataRow': {
+                    'id': dummy_id
                 },
                 'uuid': str(uuid4())
             })
-            results.append(row)
         return results
 
 
@@ -698,9 +760,10 @@ class NDTextEntity(NDBaseTool):
                 f"A line must have at least 2 points to be valid. Found {v}")
         if v['start'] < 0:
             raise ValueError(f"Text location must be positive. Found {v}")
-        if v['start'] >= v['end']:
+        if v['start'] > v['end']:
             raise ValueError(
-                f"Text start location must be less than end. Found {v}")
+                f"Text start location must be less or equal than end. Found {v}"
+            )
         return v
 
 
