@@ -3,9 +3,18 @@ from labelbox.orm.model import Field, Relationship
 
 
 class LabelingFrontend(DbObject):
-    """ Is a type representing an HTML / JavaScript UI that is used to generate
-    labels. “Image Labeling” is the default Labeling Frontend that comes in every
+    """ Label editor.
+
+    Represents an HTML / JavaScript UI that is used to generate
+    labels. “Editor” is the default Labeling Frontend that comes in every
     organization. You can create new labeling frontends for an organization.
+
+    Attributes:
+        name (str)
+        description (str)
+        iframe_url_path (str)
+
+        projects (Relationship): `ToMany` relationship to Project
     """
     name = Field.String("name")
     description = Field.String("description")
@@ -16,6 +25,15 @@ class LabelingFrontend(DbObject):
 
 
 class LabelingFrontendOptions(DbObject):
+    """ Label interface options.
+
+    Attributes:
+        customization_options (str)
+
+        project (Relationship): `ToOne` relationship to Project
+        labeling_frontend (Relationship): `ToOne` relationship to LabelingFrontend
+        organization (Relationship): `ToOne` relationship to Organization
+    """
     customization_options = Field.String("customization_options")
 
     project = Relationship.ToOne("Project")
